package jp.ac.takushoku_u.cs;

/**
* IMMethodクラスはIM法を用いて同一レベル内の配置を決定するクラスです。
* @version 1.0
* @author 笠島 裕史(Kasajima Hiroshi)
*/
/*
* 最終更新日 2005年03月02日
*/
public class IMMethod{
	
/**
* 隣接マトリクス
*/
	private BooleanMatrix matrix;
	
/**
* 可到達マトリクス
*/
	private BooleanMatrix reachMatrix;
	
/**
* ルートとする項目
*/
	private int rootItem = -1;

/**
* 要素のない隣接マトリクスを設定します。
*/
	public IMMethod(){
		matrix = new BooleanMatrix();
		reachMatrix = new BooleanMatrix();
	}
	
/**
* 隣接マトリクスからグラフを描くためのレベル(縦、横軸情報)を求めます。
* @param adjacencyMatrix 隣接マトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方でない場合
*/
	public IMMethod(BooleanMatrix adjacencyMatrix){
		matrix = new BooleanMatrix();
		reachMatrix = new BooleanMatrix();
		if(adjacencyMatrix.isSquare()){
			matrix.matrixCopy(adjacencyMatrix);
			reachMatrix.matrixCopy(matrix.getReachMatrix());
		}
		else{
			throw new NotSquareMatrixException();
		}
	}
	
	
/**
* グラフを描く際のレベルを配列として返します。
* @return レベルを格納した配列
*/
	public int[] getPaintLevel(){
		int[] returnArray = new int[matrix.getRowLength()];
		BooleanMatrix reachMatrix= new BooleanMatrix();
		reachMatrix.matrixCopy(matrix.getReachMatrix());
		
		//配列の初期化
		for(int i = 0; i < returnArray.length; i++){
			returnArray[i] = -1;
		}
		
		//ルートを決める
		if(rootItem == -1){
			for(int i = 0; i < returnArray.length; i++){
				if(matrix.getRowSum(i) == 0){
					returnArray[i] = 0;
				}
			}
		}
		else{
			returnArray[rootItem] = 0;
		}
		
		boolean flag = true;
		
		//レベルを格納する
		for(int level = 0; level < returnArray.length && flag; level++){
			for(int i = 0; i < returnArray.length; i++){
				
				//レベルが決定していなければ
				if(returnArray[i] == -1){
					int count = 1;
					for(int j = 0; j < returnArray.length; j++){
						
						//枝の先の要素はレベルが決定しているか
						if(reachMatrix.getCell(i, j) && returnArray[j] >= 0 &&
							returnArray[j] < level){
							count++;
						}
						else if(matrix.getCell(i, j) && matrix.getCell(j, i)){
							count++;
						}
					}
					if(count == reachMatrix.getRowSum(i)){
						returnArray[i] = level;
					}
				}
			}
		}
		
		//ルートの位置を変更する
		for(int i = 0; i < returnArray.length; i++){
			if(returnArray[i] == 0){
				int count = 0;
				for(int j = 0; j < matrix.getRowLength(); j++){
					if(matrix.getCell(j,i)){
						if(count < returnArray[j]){
							count = returnArray[j];
						}
					}
				}
				returnArray[i] = count - 1;
			}
		}
		
		return returnArray;
	}
	
/**
* IM法を利用して求めた各レベルでの配置順を配列として返します。
* 同一レベルでは要素の番号が小さいものほど左側に描画されます。
* @return 配置順を格納した配列
*/
	public int[] getPaintWideNumber(){
		return getPaintWideNumber(getPaintLevel());
	}
	
/**
* IM法を利用して求めた各レベルでの配置順を配列として返します。
* 同一レベルでは要素の番号が小さいものほど左側に描画されます。
* @param levelArray 描画を行う各要素のレベルを格納した配列
* @return 配置順を格納した配列
*/
	public int[] getPaintWideNumber(int[] levelArray){
		int[] returnArray = new int[matrix.getRowLength()];
		for(int i = 0; i < returnArray.length; i++){
			returnArray[i] = Integer.MAX_VALUE;//(int)Double.NaN;
		}
		
		//最大レベル
		int maxWide = getRowMaxLevel(levelArray);
		
		//現在までにそのレベルはいくつ要素を描いたか
		int[] elementWide = new int[maxWide + 1];
		for(int i = 0; i < elementWide.length; i++){
			elementWide[i] = 0;
		}
		
		//各項目の重要度を求めた配列
		double[] impotant = new double[returnArray.length];
		System.arraycopy(getImportantItemArray(matrix), 0, impotant, 0, impotant.length);
		
		//次に選択可能な項目の配列
		boolean[] nextChoice = new boolean[returnArray.length];
		
		//選択可能項目が複数あった場合の候補となる項目の配列
		boolean[] choice = new boolean[returnArray.length];
		
		//部分グラフ
		BooleanMatrix sectionGraph = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(),
			matrix.getRowNameArray(), matrix.getColNameArray());
		sectionGraph.clearMatrix();
		
		//初期選択項目
		int nextItem = getMaxItem(impotant);
		returnArray[nextItem] = 0;		//中心に配置
		elementWide[levelArray[nextItem]]++;
		
		//次の項目を選択
		int secondItem = getMaxItem(matrix, impotant, nextItem);
		returnArray[secondItem] = 0;		//中心に配置
		elementWide[levelArray[secondItem]]++;
		
		//部分グラフの作成
		if(matrix.getCell(nextItem, secondItem)){
			sectionGraph.setCell(nextItem, secondItem, true);
		}
		else{
			sectionGraph.setCell(secondItem, nextItem, true);
		}
		
		//次(3番目)に選択可能な項目の作成
		for(int i = 0; i < nextChoice.length; i++){
			
			//2要素に隣接する要素を探す
			if(i != nextItem && i != secondItem && 
				(matrix.getCell(i, nextItem) || matrix.getCell(nextItem, i) ||
					matrix.getCell(i, secondItem) || matrix.getCell(secondItem, i))){
						nextChoice[i] = true;
			}
			else{
				nextChoice[i] = false;
			}
		}
		
		//ループ(3番目の要素以降を選択)
		for(int k = 2; k < levelArray.length; k++){
			
			//重要度が最も高い項目の配列
			System.arraycopy(getMaxItemArray(nextChoice, impotant), 0, choice, 0, choice.length);
			
			//他に同じ重要度をもつ項目がないか
			int num = 0;
			if(isATrue(choice)){
				for(int i = 0; i < choice.length; i++){
					if(choice[i]){
						num = i;
					}
				}
			}
			
			//決まらない場合
			else{
				
				//部分グラフG'に隣接する要素が多い要素を選択するメソッド
				num = getChoiceElement(choice, levelArray, returnArray, sectionGraph);
			}
			
			//配置を決定する(配置する項目はnum)
			//中心に配置できる場合
			if(elementWide[levelArray[num]] == 0){
				returnArray[num] = 0;
				elementWide[levelArray[num]]++;
			}
			else{
				returnArray[num] = getPoint(num, levelArray, maxWide, returnArray, sectionGraph, matrix);
				elementWide[levelArray[num]]++;
			}
			
			//部分グラフを作る
			for(int i = 0; i < sectionGraph.getRowLength(); i++){
				if(sectionGraph.getRowSum(i) > 0 || sectionGraph.getColSum(i) > 0){
					if(matrix.getCell(i, num)){
						sectionGraph.setCell(i, num, true);
					}
					else if(matrix.getCell(num, i)){
						sectionGraph.setCell(num, i, true);
					}
				}
			}
			
			//配置した項目と隣接する項目を探す
			nextChoice[num] = false;
			for(int i = 0; i < nextChoice.length; i++){
				if(sectionGraph.getRowSum(i) > 0 || sectionGraph.getColSum(i) > 0){
					for(int j = 0; j < matrix.getRowLength(); j++){
						if(i != j && (matrix.getCell(i, j) || matrix.getCell(j, i)) &&
						(sectionGraph.getRowSum(j) == 0 && sectionGraph.getColSum(j) == 0)){
							nextChoice[j] = true;
						}
					}
				}
			}
			
		//ループ終了
		}
		
		//returnArrayを0から始まるものに変える
		System.arraycopy(getWideArray(levelArray, maxWide, returnArray), 0, returnArray, 0, returnArray.length);
		
		return returnArray;
	}
	
/**
* 最大レベルを探索します。
* @param levelArray レベルが入った配列
* @return 最大レベル
*/
	private int getRowMaxLevel(int[] levelArray){
		int num = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] > num){
				num = levelArray[i];
			}
		}
		return num;
	}
	
/**
* 指定された項目の重要度を返します。<br>
* ここでの重要度は項目itemに隣接する全ての要素iに対して、I(v_item, v_i) = N[A(v_item)]*N[R(v_i)]<br>
* (但し、v_itemとv_iが逆の場合あり)を求め、それを全て足したものになります。
* ここで、N[]はその集合の要素数、Aは先行集合、Rは可到達集合となります。
* @param baseMatrix 重要度を求めるマトリクス
* @param item 指定する項目
* @return 指定した項目の重要度
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
	public double getImportantItem(BooleanMatrix baseMatrix, int item){
		if(baseMatrix.isSquare()){
			BooleanMatrix temporaryMatrix = new BooleanMatrix(baseMatrix.getRowLength(), baseMatrix.getColLength());
			temporaryMatrix.matrixCopy(baseMatrix.getReachMatrix());
			double value = 0.0;
			
			for(int i = 0; i < baseMatrix.getRowLength(); i++){
				
				//I(v_i, v_item)を求める
				if(baseMatrix.getCell(i, item)){
					value = temporaryMatrix.getColSum(i) * temporaryMatrix.getRowSum(item) + value;
				}
				
				//I(v_item, v_i)を求める
				if(baseMatrix.getCell(item, i)){
					value = temporaryMatrix.getColSum(item) * temporaryMatrix.getRowSum(i) + value;
				}
			}
			return value;
		}
		else{
			throw new NotSquareMatrixException();
		}
	}
	
/**
* 全項目の重要度を配列で返します。
* @param baseMatrix 重要度を求めるマトリクス
* @return 重要度を格納した配列
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
	public double[] getImportantItemArray(BooleanMatrix baseMatrix){
		if(baseMatrix.isSquare()){
			double[] returnArray = new double[baseMatrix.getRowLength()];
			for(int i = 0; i < baseMatrix.getRowLength(); i++){
				returnArray[i] = getImportantItem(baseMatrix, i);
			}
			return returnArray;
		}
		else{
			throw new NotSquareMatrixException();
		}
	}
	
/**
* 配列の中身が最大である要素の番号を返します。
* @param lookArray 探索する配列
* @return 最大の項目番号
*/
	private int getMaxItem(double[] lookArray){
		int value = 0;
		double max;
		max = lookArray[0];
		for(int i = 1; i < lookArray.length; i++){
			if(max < lookArray[i]){
				max = lookArray[i];
				value = i;
			}
		}
		return value;
	}
	
/**
* 指定した要素と隣接なもののうち配列の中身が最大である要素の番号を返します。
* @param baseMatrix 隣接マトリクス
* @param lookArray 探索する配列
* @param item 指定する項目
* @return 最大の項目番号
*/
	private int getMaxItem(BooleanMatrix baseMatrix, double[] lookArray, int item){
		int value = 0;
		double max;
		max = 0.0;
		for(int i = 0; i < lookArray.length; i++){
			if(i != item && max < lookArray[i] &&(baseMatrix.getCell(i, item) || baseMatrix.getCell(item, i))){
				max = lookArray[i];
				value = i;
			}
		}
		return value;
	}
	
/**
* 選択可能項目のうち、配列の中身が最大である要素候補を配列で返します。
* @param choiceArray 選択可能項目を示した配列
* @param lookArray 探索する配列
* @return 最大となる候補を格納した配列
*/
	private boolean[] getMaxItemArray(boolean[] choiceArray, double[] lookArray){
		boolean[] returnArray = new boolean[choiceArray.length];
		double max = 0.0;
		
		//最大値の探索
		for(int i = 0; i < lookArray.length; i++){
			if(choiceArray[i] && lookArray[i] > max){
				max = lookArray[i];
			}
		}
		
		//最大値を全てtrueにする
		for(int i = 0; i < lookArray.length; i++){
			if(choiceArray[i] && lookArray[i] == max){
				returnArray[i] = true;
			}
			else{
				returnArray[i] = false;
			}
		}
		
		return returnArray;
	}
	
/**
* 論理型配列でtrueの個数が1つかどうか探索します。
* @param lookArray 探索する配列
* @return trueが1つであれば、 true
*/
	private boolean isATrue(boolean[] lookArray){
		int count = 0;
		for(int i = 0; i < lookArray.length; i++){
			if(lookArray[i]){
				count++;
			}
		}
		
		return count == 1;
	}
	
/**
* 選択できる要素が複数ある場合に選択する要素を選びます。
* @param choiceArray 選択候補となる要素を示した配列
* @param levelArray 各要素のレベルが格納された配列
* @param wideArray 既に配置が決定した配列
* @param partMatrix 部分グラフ
* @return 要素番号
*/
	private int getChoiceElement(boolean[] choiceArray, int[] levelArray, int[] wideArray, BooleanMatrix partMatrix){
		
		//1.中心軸に近く配置できる要素を選択する
		int number = -1;
		int value = Integer.MAX_VALUE;
		for(int i =0; i < choiceArray.length; i++){
			if(choiceArray[i]){
				if(getArrangePoint(i, levelArray, wideArray) < value){
					number = i;
					value = getArrangePoint(i, levelArray, wideArray);
				}
			}
		}
		
		//非選択項目を排除する
		for(int i = 0; i < choiceArray.length; i++){
			if(choiceArray[i] && getArrangePoint(i, levelArray, wideArray) > value){
				choiceArray[i] = false;
			}
		}
		
		//選択可能項目は1つかどうか
		if(!isATrue(choiceArray)){
		
			//2.部分グラフに隣接する要素が多いものを選択する
			//3.いずれかの要素を選択する(この場合は要素番号の小さいもの)
			number = 0;
			value = 0;
			for(int i =0; i < choiceArray.length; i++){
				
				//隣接している個数を数える
				int count = 0;
				if(choiceArray[i]){
					for(int j = 0; j < choiceArray.length; j++){
						if(1 != j &&(matrix.getCell(i , j) || matrix.getCell(j, i))){
							boolean flag = true;
							for(int k = 0; k < choiceArray.length && flag; k++){
								if(partMatrix.getCell(j ,k) || partMatrix.getCell(k ,j)){
									flag = false;
								}
							}
							if(!flag){
								count++;
							}
						}
					}
				}
				if(count > value){
					number = i;
					value = count;
				}
				
			}
		}
		else{
			for(int i = 0;i < choiceArray.length; i++){
				if(choiceArray[i]){
					number = i;
				}
			}
		}
		
		return number;
	}
	
/**
* 指定した要素が中心軸からいくつめに配置できるかを返します。
* @param item 指定する要素
* @param levelArray 各要素のレベルが格納された配列
* @param wideArray 既に配置が決定した配列
* @return 配置できる番号(中心軸なら0)
*/
	private int getArrangePoint(int item, int[] levelArray, int[] wideArray){
		int max = 0;
		int min = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(i != item && levelArray[i] == levelArray[item]){
				if(wideArray[i] != Integer.MAX_VALUE){
					if(wideArray[i] >= 0){
						if(wideArray[i] > max){
							max = wideArray[i];
						}
					}
					else{
						if(wideArray[i] < min){
							min = wideArray[i];
						}
					}
				}
			}
		}
		
		if(Math.abs(max) > Math.abs(min)){
			return Math.abs(min);
		}
		else{
			return Math.abs(max);
		}
	}
	
/**
* 中心軸に配置できない場合右端左端のどちらに配置するか決定します。
* @param item 配置を行う要素
* @param levelArray 各要素のレベルが格納された配列
* @param maxLevel 最大レベル数
* @param wideArray 既に配置が決定した配列
* @param partMatrix 部分グラフ
* @param graph 元のグラフ
* @return 描画を行う場所(負ならば左、正ならば右)
*/
	private int getPoint(int item, int[] levelArray, int maxLevel, int[] wideArray, BooleanMatrix partMatrix, BooleanMatrix graph){
		int returnValue;
		int left = 0;
		int right = 0;
		
		//1.G'の最左端または最右端の要素と隣接する要素が多いほうに配置する
		for(int i = 0; i <= maxLevel; i++){
			int max = 0;
			int maxElement = 0;
			int min = 0;
			int minElement = 0;
			
			//最右端と最左端の探索
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == i && wideArray[j] != Integer.MAX_VALUE &&
				(partMatrix.getRowSum(j) > 0 || partMatrix.getColSum(j) > 0)){
					if(wideArray[j] >= max){
						max = wideArray[j];
						maxElement = j;
					}
					if(wideArray[j] <= min){
						min = wideArray[j];
						minElement = j;
					}
				}
				
			}
			
			//部分グラフと隣接しているか
			if((graph.getCell(item, maxElement) || graph.getCell(maxElement, item))&&
			(partMatrix.getRowSum(maxElement) > 0 || partMatrix.getColSum(maxElement) > 0)){
				right++;
			}
			
			if((graph.getCell(item, minElement) || graph.getCell(minElement, item)) && 
			(partMatrix.getRowSum(minElement) > 0 || partMatrix.getColSum(minElement) > 0)){
				left++;
			}
		}
		
		//左の方が多いので左に配置
		if(left > right){
			int min = 0;
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
					if(wideArray[j] < min){
						min = wideArray[j];
					}
				}
			}
			returnValue = min - 1;
		}
		
		//右の方が多いので右に配置
		else if(right > left){
			int max = 0;
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
					if(wideArray[j] > max){
						max = wideArray[j];
					}
				}
			}
			returnValue = max + 1;
		}
		
		//2.同じ場合G'へ有向枝を描画したとき右と左で枝の長さの総和が小さいほうに配置する
		else{
			left = 0;
			right = 0;
			
			for(int i = 0; i < levelArray.length; i++){
				if((graph.getCell(item, i) || graph.getCell(i, item)) &&
				(partMatrix.getRowSum(i) > 0)){
					
					if(wideArray[i] >= 0){
							right = right + Math.abs(levelArray[i] - levelArray[item]);
						}
						else if(wideArray[i] <= 0){
							left = left +  Math.abs(levelArray[i] - levelArray[item]);
						}
					
				}
			}
			
			//左の方が小さいので左に配置
			if(left < right){
				int min = 0;
				for(int j = 0; j < levelArray.length; j++){
					if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
						if(wideArray[j] < min){
							min = wideArray[j];
						}
					}
				}
				returnValue = min - 1;
			}
			
			//右の方が小さいので右に配置
			else if(right < left){
				int max = 0;
				for(int j = 0; j < levelArray.length; j++){
					if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
						if(wideArray[j] > max){
							max = wideArray[j];
						}
					}
				}
				returnValue = max + 1;
			}
			
			//3.G'の右と左で少ない要素の側に配置する
			else{
				right = 0;
				left = 0;
				for(int i = 0; i < levelArray.length; i++){
					if((partMatrix.getRowSum(i) > 0 || partMatrix.getColSum(i) > 0) 
						&& wideArray[i] != Integer.MAX_VALUE){
						if(wideArray[i] > 0){
							right++;
						}
						else if(wideArray[i] < 0){
							left++;
						}
					}
				}
				
				//右側の要素が多いので左に配置
				if(left < right){
					int min = 0;
					for(int j = 0; j < levelArray.length; j++){
						if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
							if(wideArray[j] < min){
								min = wideArray[j];
							}
						}
					}
					returnValue = min - 1;
				}
				
				//左側の要素が多いので右に配置
				else if(right < left){
					int max = 0;
					for(int j = 0; j < levelArray.length; j++){
						if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
							if(wideArray[j] > max){
								max = wideArray[j];
							}
						}
					}
					returnValue = max + 1;
				}
				
				//4.右か左かいずれかに配置する。この場合は右
				else{
					//returnValue = 1;
					int max = 0;
					for(int j = 0; j < levelArray.length; j++){
						if(levelArray[j] == levelArray[item] && wideArray[j] != Integer.MAX_VALUE){
							if(wideArray[j] > max){
								max = wideArray[j];
							}
						}
					}
					returnValue = max + 1;
				}
			}
		}
		
		return returnValue;
	}
	
/**
* 同一レベルの配置を1から始まる配列として返します。
* ただし、中心軸を境に左右対称となるように配置します。
* このため1がない場合もあります。(右側が多い場合)
* @param levelArray 各要素のレベルが格納された配列
* @param maxLevel 最大レベル数
* @param wideArray 既に配置が決定した配列
* @retrun 既に配置が決定した配列を修正した配列
*/
	private int[] getWideArray(int[] levelArray, int maxLevel, int[] wideArray){
		
		
		for(int i = 0; i <= maxLevel; i++){
			
			//各レベルの最小値、最大値を探す
			int min = Integer.MAX_VALUE;
			int max = Integer.MIN_VALUE;
			
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == i){
					if(wideArray[j] < min){
						min = wideArray[j];
					}
					if(wideArray[j] > max){
						max = wideArray[j];
					}
				}
				
			}
			
			//左右が等しいとき
			//左が多いとき
			if(Math.abs(max) <= Math.abs(min)){
				for(int j = 0; j < levelArray.length; j++){
					if(levelArray[j] == i){
						wideArray[j] = wideArray[j] + Math.abs(min) + 1;
					}
				}
			}
			//右が多いとき
			else if(Math.abs(max) > Math.abs(min)){
				int sa = Math.abs(max) - Math.abs(min);
				for(int j = 0; j < levelArray.length; j++){
					if(levelArray[j] == i){
						wideArray[j] = wideArray[j] + Math.abs(min) + sa + 1;
					}
				}
			}
		}
		
		return wideArray;
	}
	
}